import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity

sample_data = {
    "post_id": [1, 1, 1, 2, 2, 3, 3],
    "comment_id": [1, 2, 3, 4, 5, 6, 7],
    "comment": ["This is good", "Amazing!", "Good work", "Hello there!", "Hi", "Nice one", "Nice!"],
    "num_comments": [3, 3, 3, 2, 2, 2, 2],
    "score": [10, 15, 5, 8, 12, 3, 6],
}
df = pd.DataFrame(sample_data)
df = df.sort_values(by="score", ascending=False)
df["word_count"] = df["comment"].apply(lambda x: len(x.split()))
df = df[df["word_count"] >= 3]
post_counts = df["post_id"].value_counts()
valid_posts = post_counts[post_counts >= 2].index
df = df[df["post_id"].isin(valid_posts)]

sampled_comments = df["comment"].sample(n=min(500000, len(df)), random_state=42)
tfidf_vectorizer = TfidfVectorizer()
tfidf_matrix = tfidf_vectorizer.fit_transform(sampled_comments)

DUPLICATE_THRESH = 0.6 # 0.6 for reddit 0.7 for youtube
result_rows = []

for post_id, group in df.groupby("post_id"):
    comments = group["comment"].tolist()
    scores = group["score"].tolist()
    comment_ids = group["comment_id"].tolist()
    tfidf_vectors = tfidf_vectorizer.transform(comments)
    similarities = cosine_similarity(tfidf_vectors)
    keep_indices = set(range(len(comments)))  # Start with all indices
    for i in range(len(comments)):
        for j in range(i + 1, len(comments)):
            if similarities[i, j] > DUPLICATE_THRESH:
                if scores[i] >= scores[j]:
                    keep_indices.discard(j)
                else:
                    keep_indices.discard(i)

    for idx in keep_indices:
        result_rows.append({
            "post_id": post_id,
            "comment_id": comment_ids[idx],
            "comment": comments[idx],
            "score": scores[idx],
        })

result_df = pd.DataFrame(result_rows)
print(result_df)
